from __future__ import absolute_import
from __future__ import unicode_literals
from markdown import Extension
from markdown.blockprocessors import BlockProcessor
from markdown.util import etree
import re

from pelicanconf import Q_AND_A_CLASSES


class QandAExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.parser.blockprocessors.add('q-and-a',
                                      QandAProcessor(md.parser),
                                      '_begin')


class QandAProcessor(BlockProcessor):
    statements = r'|'.join(Q_AND_A_CLASSES)
    RE = re.compile(r'(.*)\s*?(%s)\s*?' % statements)

    def test(self, parent, block):
        # [('How do I update my system?', '!!T')]
        matches = self.RE.search(block)
        if matches:
            return matches
        return False

    def run(self, parent, blocks):
        block = blocks.pop(0)
        matches = self.RE.search(block)
        statement = matches.group(2)
        block = block.replace(statement, '')
        question = block.split('\n')[0]
        answer = '\n'.join(block.split('\n')[1:])

        if statement == '!!T':
            # <div class="row">
            # <div class="eight-col last-col">
            # <!-- question & answer -->
            # </div>
            # </div>
            div = etree.SubElement(parent, 'div')
            div.set('class', 'row')
            div2 = etree.SubElement(div, 'div')
            div2.set('class', 'eight-col last-col')
            self.parser.parseChunk(div2, question)
            self.parser.parseChunk(div2, answer)
        elif statement == '!!I':
            # <div class="row">
            # <div class="eight-col">
            # <!-- question & answer -->
            # </div>
            # <div class="four-col last-col">
            # <!-- question & answer -->
            # </div>
            # </div>
            image_block = "..."
            # XXX: we need to figure out how to parse if an image is
            # there, etc.
            div = etree.SubElement(parent, 'div')
            div.set('class', 'row')
            text_div = etree.SubElement(div, 'div')
            text_div.set('class', 'eight-col')
            img_div = etree.SubElement(div, 'div')
            img_div.set('class', 'four-col last-col')
            self.parser.parseChunk(text_div, question)
            self.parser.parseChunk(text_div, answer)
            self.parser.parseChunk(img_div, image_block)
        else:
            sibling = self.lastChild(parent)
            self.parser.parseChunk(sibling, block)


def makeExtension(*args, **kwargs):
    return QandAExtension(*args, **kwargs)
