import codecs
from html.parser import HTMLParser
import os
import shutil
import subprocess
import tempfile
from unittest import TestCase
import urllib.parse


def require_build(build):
    tempdir = tempfile.mkdtemp()
    env = {}
    if build == 'html':
        env = {'OUTPUTDIR': tempdir}
    if build == 'web':
        env = {'OUTPUTDIR_WEB': tempdir}
    ret = subprocess.call(['make', '-es', build], env=env)
    return (ret, tempdir)


def clean_tempdir(tempdir):
    if os.path.exists(tempdir):
        shutil.rmtree(tempdir)


class MyHTMLParser(HTMLParser):
    links = []

    def handle_starttag(self, tag, attrs):
        if tag == "a":
            for name, value in attrs:
                if name == "href":
                    url = urllib.parse.urlparse(value)
                    if not value.startswith('#') and \
                       url.netloc in [None, '', 'localhost']:
                        self.links += [value]

    def reload(self):
        links = self.links
        self.links = []
        return links


class HelpTestCase(TestCase):
    def __init__(self, *args):
        self.pwd = os.getcwd()
        self.htmlparser = MyHTMLParser()
        TestCase.__init__(self, *args)

    def __del__(self):
        os.chdir(self.pwd)

    def _test_local_links(self, build):
        (ret, tempdir) = require_build(build)
        if ret:
            return False
        for dirpath, dirnames, filenames in os.walk(tempdir):
            for fn in filenames:
                full_fn = os.path.join(dirpath, fn)
                os.chdir(dirpath)
                if full_fn.endswith('.html'):
                    html = codecs.open(full_fn, encoding='utf-8').read()
                    self.htmlparser.feed(html)
                links = self.htmlparser.reload()
                for link in links:
                    rel_path = os.path.relpath(link, full_fn)
                    path = os.path.normpath(os.path.join(full_fn, rel_path))
                    self.assertTrue(os.path.exists(path))
        clean_tempdir(tempdir)
        return True

    def test_local_phone_links(self):
        return self._test_local_links('html')

    def test_local_web_links(self):
        return self._test_local_links('web')
